@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: alias.c,v 2.0 1998/12/12 08:05:43 roessler Rel $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -153,6 +152,40 @@ static void write_safe_address (FILE *fp, char *s)
   }
 }
 
+ADDRESS *mutt_get_address (ENVELOPE *env, char **pfxp)
+{
+  ADDRESS *adr;
+  char *pfx = NULL;
+
+  if (mutt_addr_is_user (env->from))
+  {
+    if (env->to && !mutt_is_mail_list (env->to))
+    {
+      pfx = "To";
+      adr = env->to;
+    }
+    else
+    {
+      pfx = "Cc";
+      adr = env->cc;
+    }
+  }
+  else if (env->reply_to && !mutt_is_mail_list (env->reply_to))
+  {
+    pfx = "Reply-To";
+    adr = env->reply_to;
+  }
+  else
+  {
+    adr = env->from;
+    pfx = "From";
+  }
+
+  if (pfxp) *pfxp = pfx;
+
+  return adr;
+}
+
 void mutt_create_alias (ENVELOPE *cur, ADDRESS *iadr)
 {
   ALIAS *new, *t;
@@ -162,18 +195,7 @@ void mutt_create_alias (ENVELOPE *cur, ADDRESS *iadr)
 
   if (cur)
   {
-    if (mutt_addr_is_user (cur->from))
-    {
-      if (cur->to && !mutt_is_mail_list (cur->to))
-	adr = cur->to;
-      else
-	adr = cur->cc;
-    }
-    else if (cur->reply_to && !mutt_is_mail_list (cur->reply_to))
-      adr = cur->reply_to;
-    else
-      adr = cur->from;
-
+    adr = mutt_get_address (cur, NULL);
   }
   else if (iadr)
   {
@@ -297,7 +319,8 @@ ADDRESS *alias_reverse_lookup (ADDRESS *a)
 /* alias_complete() -- alias completion routine
  *
  * given a partial alias, this routine attempts to fill in the alias
- * from the alias list as much as possible
+ * from the alias list as much as possible. if given empty search string
+ * or found nothing, present all aliases
  */
 int mutt_alias_complete (char *s, size_t buflen)
 {
@@ -306,36 +329,38 @@ int mutt_alias_complete (char *s, size_t buflen)
   char bestname[STRING];
   int i;
 
-  memset (bestname, 0, sizeof (bestname));
-
 #define min(a,b)        ((a<b)?a:b)
 
-  while (a)
+  if (s[0] != 0) /* avoid empty string as strstr argument */
   {
-    if (a->name && strstr (a->name, s) == a->name)
-    {
-      if (!bestname[0]) /* init */
-	strfcpy (bestname, a->name, min (mutt_strlen (a->name) + 1, sizeof (bestname)));
-      else
-      {
-	for (i = 0 ; a->name[i] && a->name[i] == bestname[i] ; i++)
-	  ;
-	bestname[i] = 0;
-      }
-    }
-    a = a->next;
-  }
+    memset (bestname, 0, sizeof (bestname));
 
-  if ((bestname[0] == 0) || /* if we didn't find anything */
-      (s[0] == 0))          /* or we weren't given anything */
-  {
-    mutt_alias_menu (s, buflen, Aliases);
-    return 0;
-  }
-  else
-  {
-    if (mutt_strcmp (bestname, s) == 0) /* add anything to the completion? */
+    while (a)
     {
+      if (a->name && strstr (a->name, s) == a->name)
+      {
+	if (!bestname[0]) /* init */
+	  strfcpy (bestname, a->name,
+		   min (mutt_strlen (a->name) + 1, sizeof (bestname)));
+	else
+	{
+	  for (i = 0 ; a->name[i] && a->name[i] == bestname[i] ; i++)
+	    ;
+	  bestname[i] = 0;
+	}
+      }
+      a = a->next;
+    }
+
+    if (bestname[0] != 0)
+    {
+      if (mutt_strcmp (bestname, s) != 0)
+      {
+	/* we are adding something to the completion */
+	strfcpy (s, bestname, mutt_strlen (bestname) + 1);
+	return 1;
+      }
+
       /* build alias list and show it */
       a = Aliases;
       while (a)
@@ -354,25 +379,23 @@ int mutt_alias_complete (char *s, size_t buflen)
 	}
 	a = a->next;
       }
-
-      s[0] = 0; /* reset string before passing to alias_menu */
-      mutt_alias_menu (s, buflen, a_list);
-
-      /* free the alias list */
-      while (a_list)
-      {
-	a_cur = a_list;
-	a_list = a_list->next;
-	safe_free ((void **) &a_cur);
-      }
-
-      return 0;
     }
-    else /* we are adding something to the completion */
-      strfcpy (s, bestname, mutt_strlen (bestname) + 1);
   }
 
-  return 1;
+  bestname[0] = 0;
+  mutt_alias_menu (bestname, sizeof(bestname), a_list ? a_list : Aliases);
+  if (bestname[0] != 0)
+    strfcpy (s, bestname, buflen);
+
+  /* free the alias list */
+  while (a_list)
+  {
+    a_cur = a_list;
+    a_list = a_list->next;
+    safe_free ((void **) &a_cur);
+  }
+
+  return 0;
 }
 
 static int string_is_address(const char *str, const char *u, const char *d)
@@ -397,11 +420,14 @@ int mutt_addr_is_user (ADDRESS *addr)
 
   if (mutt_strcasecmp (addr->mailbox, Username) == 0)
     return 1;
-  if(string_is_address(addr->mailbox, Username, Hostname))
+  if (string_is_address(addr->mailbox, Username, Hostname))
     return 1;
-  if(string_is_address(addr->mailbox, Username, mutt_fqdn(0)))
+  if (string_is_address(addr->mailbox, Username, mutt_fqdn(0)))
     return 1;
-  if(string_is_address(addr->mailbox, Username, mutt_fqdn(1)))
+  if (string_is_address(addr->mailbox, Username, mutt_fqdn(1)))
+    return 1;
+
+  if (From && !mutt_strcasecmp (From->mailbox, addr->mailbox))
     return 1;
 
   if (Alternates.pattern &&
