@@ -47,30 +47,35 @@
 #include <sys/stat.h>
 #include <errno.h>
 
+static const char No_mailbox_is_open[] = N_("No mailbox is open.");
+static const char There_are_no_messages[] = N_("There are no messages.");
+static const char Mailbox_is_read_only[] = N_("Mailbox is read-only.");
+static const char Function_not_permitted_in_attach_message_mode[] = N_("Function not permitted in attach-message mode.");
+
 #define CHECK_MSGCOUNT if (!Context) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _("No mailbox is open."); \
+		mutt_error _(No_mailbox_is_open); \
 		break; \
 	} \
 	else if (!Context->msgcount) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _("There are no messages."); \
+		mutt_error _(There_are_no_messages); \
 		break; \
 	}
 
 #define CHECK_READONLY if (Context->readonly) \
 			{ \
 			  	mutt_flushinp (); \
-				mutt_error _("Mailbox is read-only."); \
+				mutt_error _(Mailbox_is_read_only); \
 				break; \
 			}
 
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
 		     {\
 			mutt_flushinp (); \
-			mutt_error ("Function not permitted in attach-message mode."); \
+			mutt_error _(Function_not_permitted_in_attach_message_mode); \
 			break; \
 		     }
 
@@ -863,17 +868,17 @@ int mutt_index_menu (void)
 
       case OP_MAIN_CHANGE_FOLDER:
       
-	if (attach_msg || option (OPTREADONLY))
+	if (attach_msg)
 	  op = OP_MAIN_CHANGE_FOLDER_READONLY;
 
 	/* fallback to the readonly case */
 
       case OP_MAIN_CHANGE_FOLDER_READONLY:
 
-        if (op == OP_MAIN_CHANGE_FOLDER)
-          cp = _("Open mailbox");
-        else
+        if ((op == OP_MAIN_CHANGE_FOLDER_READONLY) || option (OPTREADONLY))
           cp = _("Open mailbox in read-only mode");
+        else
+          cp = _("Open mailbox");
 
 	buf[0] = '\0';
 	mutt_buffy (buf);
@@ -957,7 +962,7 @@ int mutt_index_menu (void)
 	}
 
 	menu->menu = MENU_PAGER;
-/* 	menu->oldcurrent = menu->current;  */
+ 	menu->oldcurrent = menu->current;
 	continue;
 
       case OP_EXIT:
@@ -1435,11 +1440,15 @@ int mutt_index_menu (void)
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_tag_set_flag (M_TAG, 0);
 	  menu->redraw = REDRAW_INDEX;
 	}
 	else
 	{
 	  mutt_set_flag (Context, CURHDR, M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_set_flag (Context, CURHDR, M_TAG, 0);
 	  if (option (OPTRESOLVE))
 	  {
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
@@ -1482,7 +1491,7 @@ int mutt_index_menu (void)
       case OP_DISPLAY_ADDRESS:
 
 	CHECK_MSGCOUNT;
-	mutt_display_address (CURHDR->env->from);
+	mutt_display_address (CURHDR->env);
 	break;
 
       case OP_ENTER_COMMAND:
@@ -1722,7 +1731,9 @@ int mutt_index_menu (void)
     {
       menu->menu = MENU_MAIN;
       menu->redraw = REDRAW_FULL;
+#if 0
       set_option (OPTWEED); /* turn header weeding back on. */
+#endif
     }
 
     if (done) break;
