@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: edit.c,v 2.0 1998/12/12 08:06:07 roessler Rel $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -34,7 +33,7 @@ static const char rcsid[]="$Id: edit.c,v 2.0 1998/12/12 08:06:07 roessler Rel $"
  * SLcurses_waddnstr() can't take a "const char *", so this is only
  * declared "static" (sigh)
  */
-static char EditorHelp[] = N_("\
+static char* EditorHelp = N_("\
 ~~		insert a line begining with a single ~\n\
 ~b users	add users to the Bcc: field\n\
 ~c users	add users to the Cc: field\n\
