@@ -1,4 +1,3 @@
-/* $Id: keymap.h,v 2.0 1998/12/12 08:06:34 roessler Rel $ */
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -59,16 +58,19 @@ enum
   MENU_MAIN,
   MENU_PAGER,
   MENU_POST,
+  MENU_QUERY,
 
+  
 #ifdef _PGPPATH
   MENU_PGP,
 #endif
 
+#ifdef MIXMASTER
+  MENU_MIX,
+#endif
 
 
 
-
-  MENU_QUERY,
   MENU_MAX
 };
 
@@ -103,6 +105,10 @@ extern struct binding_t OpQuery[];
 extern struct binding_t OpPgp[];
 #endif /* _PGPPATH */
 
+#ifdef MIXMASTER
+extern struct binding_t OpMix[];
+#endif
+
 #include "keymap_defs.h"
 
 #endif /* KEYMAP_H */
