@@ -279,7 +279,9 @@ pid_t mutt_create_filter_fd (const char *, FILE **, FILE **, FILE **, int, int,
 
 ADDRESS *alias_reverse_lookup (ADDRESS *);
 
-
+/* base64.c */
+void mutt_to_base64 (unsigned char*, const unsigned char*, int);
+int mutt_from_base64 (char*, const char*);
 
 #ifdef LOCALES_HACK
 #define IsPrint(c) (isprint((unsigned char)(c)) || \
