@@ -338,7 +338,7 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
   if (option (OPTFORWDECODE))
   {
     cmflags |= M_CM_DECODE | M_CM_CHARCONV;
-    if (option (OPTFORWWEEDHEADER))
+    if (option (OPTWEED))
     {
       chflags |= CH_WEED | CH_REORDER;
       cmflags |= M_CM_WEED;
@@ -355,9 +355,8 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
 static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
 {
   char buffer[STRING];
-  int flags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
-
-
+  int cmflags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
+  int chflags = CH_DECODE;
 
 #ifdef _PGPPATH
   if (cur->pgp)
@@ -370,8 +369,6 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
   }
 #endif /* _PGPPATH */
 
-
-
   if (Attribution)
   {
     mutt_make_string (buffer, sizeof (buffer), Attribution, ctx, cur);
@@ -379,9 +376,15 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
     fputc ('\n', out);
   }
   if (!option (OPTHEADER))
-    flags |= M_CM_NOHEADER;
+    cmflags |= M_CM_NOHEADER;
+  if (option (OPTWEED))
+  {
+    chflags |= CH_WEED;
+    cmflags |= M_CM_WEED;
+  }
+
   mutt_parse_mime_message (ctx, cur);
-  mutt_copy_message (out, ctx, cur, flags, CH_DECODE);
+  mutt_copy_message (out, ctx, cur, cmflags, chflags);
   if (PostIndentString)
   {
     mutt_make_string (buffer, sizeof (buffer), PostIndentString, ctx, cur);
@@ -577,22 +580,20 @@ envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
       env->subject = safe_strdup ("Re: your mail");
 
     /* add the In-Reply-To field */
-    if (InReplyTo)
+    snprintf (buffer, sizeof (buffer), "In-Reply-To: %s", 
+	      cur->env->message_id);
+
+    tmp = env->userhdrs;
+    while (tmp && tmp->next)
+      tmp = tmp->next;
+    if (tmp)
     {
-      strfcpy (buffer, "In-Reply-To: ", sizeof (buffer));
-      mutt_make_string (buffer + 13, sizeof (buffer) - 13, InReplyTo, ctx, cur);
-      tmp = env->userhdrs;
-      while (tmp && tmp->next)
-	tmp = tmp->next;
-      if (tmp)
-      {
-	tmp->next = mutt_new_list ();
-	tmp = tmp->next;
-      }
-      else
-	tmp = env->userhdrs = mutt_new_list ();
-      tmp->data = safe_strdup (buffer);
+      tmp->next = mutt_new_list ();
+      tmp = tmp->next;
     }
+    else
+      tmp = env->userhdrs = mutt_new_list ();
+    tmp->data = safe_strdup (buffer);
 
     if(tag)
     {
@@ -642,6 +643,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 
     if (i == M_YES)
     {
+      mutt_message _("Including quoted message...");
       if (!cur)
       {
 	for (i = 0; i < ctx->vcount; i++)
@@ -660,14 +662,17 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
       }
       else
 	include_reply (ctx, cur, tempfp);
+
     }
   }
   else if (flags & SENDFORWARD)
   {
-    if (query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?")))
+    if ((i = query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?"))) == M_YES)
     {
       BODY *last = msg->content;
 
+      mutt_message _("Preparing forwarded message...");
+      
       while (last && last->next)
 	last = last->next;
 
@@ -697,7 +702,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	}
       }
     }
-    else
+    else if (i != -1)
     {
       if (cur)
 	include_forward (ctx, cur, tempfp);
@@ -706,6 +711,8 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	  if (ctx->hdrs[ctx->v2r[i]]->tagged)
 	    include_forward (ctx, ctx->hdrs[ctx->v2r[i]], tempfp);
     }
+    else if (i == -1)
+      return -1;
   }
 
 
@@ -722,7 +729,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
   }
 #endif
 
-
+  mutt_clear_error ();
 
   return (0);
 }
@@ -782,20 +789,30 @@ static ADDRESS *set_reverse_name (ENVELOPE *env)
   return (tmp);
 }
 
-static ADDRESS *mutt_default_from (void)
+ADDRESS *mutt_default_from (void)
 {
-  ADDRESS *adr = rfc822_new_address ();
+  ADDRESS *adr;
   const char *fqdn = mutt_fqdn(1);
-  
-  /* don't set realname here, it will be set later */
 
-  if (option (OPTUSEDOMAIN))
+  /* 
+   * Note: We let $from override $realname here.  Is this the right
+   * thing to do? 
+   */
+
+  if (From)
+    adr = rfc822_cpy_adr_real (From);
+  else if (option (OPTUSEDOMAIN))
   {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_malloc (mutt_strlen (Username) + mutt_strlen (fqdn) + 2);
     sprintf (adr->mailbox, "%s@%s", NONULL(Username), NONULL(fqdn));
   }
   else
+  {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_strdup (NONULL(Username));
+  }
+  
   return (adr);
 }
 
@@ -877,6 +894,7 @@ ci_send_message (int flags,		/* send mode */
 
 #ifdef _PGPPATH
   BODY *save_content = NULL;
+  BODY *clear_content = NULL;
   char *pgpkeylist = NULL;
   /* save current value of "pgp_sign_as" */
   char *signas = NULL;
@@ -1212,42 +1230,36 @@ main_loop:
    * one place in order to avoid going to main_loop with encoded "env"
    * in case of error.  Ugh.
    */
+
 #ifdef _PGPPATH
   if (msg->pgp)
   {
     /* save the decrypted attachments */
-    save_content = msg->content;
+    clear_content = msg->content;
 
     if ((pgp_get_keys (msg, &pgpkeylist) == -1) ||
 	(pgp_protect (msg, pgpkeylist) == -1))
     {
-      if (msg->content->parts)
-      {
-	/* remove the toplevel multipart structure */
-	pbody = msg->content;
-	msg->content = msg->content->parts;
-	pbody->parts = NULL;
-	mutt_free_body (&pbody);
-      }
+      msg->content = mutt_remove_multipart (msg->content);
+      
       if (pgpkeylist)
 	FREE (&pgpkeylist);
+
       goto main_loop;
     }
   }
+
+  /* 
+   * at this point, msg->content is one of the following three things:
+   * - multipart/signed.  In this case, clear_content is a child.
+   * - multipart/encrypted.  In this case, clear_content exists
+   *   independently
+   * - application/pgp.  In this case, clear_content exists independently.
+   * - something else.  In this case, it's the same as clear_content.
+   */
+
 #endif /* _PGPPATH */
 
-  /* the following check may _badly_ interact with the PGP code above. */
-  
-#if 0
-  if (flags & SENDEDITMSG)
-  {
-   int really_send = mutt_yesorno (_("Message edited. Really send?"), 1);
-   if (really_send != M_YES)
-     goto main_loop;
-  }
-#endif
-
-
   if (!option (OPTNOCURSES) && !(flags & SENDMAILX))
     mutt_message _("Sending message...");
 
@@ -1272,17 +1284,17 @@ main_loop:
       if (mutt_strcmp (msg->content->subtype, "encrypted") == 0 ||
 	  mutt_strcmp (msg->content->subtype, "signed") == 0)
       {
-	if (save_content->type == TYPEMULTIPART)
+	if (clear_content->type == TYPEMULTIPART)
 	{
 	  if (!(msg->pgp & PGPENCRYPT) && (msg->pgp & PGPSIGN))
 	  {
 	    /* save initial signature and attachments */
 	    save_sig = msg->content->parts->next;
-	    save_parts = msg->content->parts->parts->next;
+	    save_parts = clear_content->parts->next;
 	  }
 
 	  /* this means writing only the main part */
-	  msg->content = save_content->parts;
+	  msg->content = clear_content->parts;
 
 	  if (pgp_protect (msg, pgpkeylist) == -1)
 	  {
@@ -1294,17 +1306,7 @@ main_loop:
 	    goto full_fcc;
 	  }
 
-	  if (msg->pgp & PGPENCRYPT)
-	  {
-	    /* not released in pgp_encrypt_message() */
-	    mutt_free_body (&save_content->parts);
-	    /* make sure we release the right thing later */
-	    save_content->parts = msg->content;
-
-	    encode_descriptions (msg->content);
-	  }
-	  else
-	    save_content = msg->content;
+	  save_content = msg->content;
 	}
       }
       else
@@ -1317,6 +1319,7 @@ full_fcc:
 #endif /* _PGPPATH */
     if (msg->content)
       mutt_write_fcc (fcc, msg, NULL, 0, NULL);
+
     msg->content = tmpbody;
 
 #ifdef _PGPPATH
@@ -1331,22 +1334,33 @@ full_fcc:
       msg->content->parts->next = save_sig;
       msg->content->parts->parts->next = save_parts;
     }
+    else if (save_content)
+    {
+      /* destroy the new encrypted body. */
+      mutt_free_body (&save_content);
+    }
+      
 #endif /* _PGPPATH */
   }
 
-#ifdef _PGPPATH
-  if (msg->pgp & PGPENCRYPT)
-  {
-    /* cleanup structures from the first encryption */
-    mutt_free_body (&save_content);
-    FREE (&pgpkeylist);
-  }
-#endif /* _PGPPATH */
 
   if ((i = send_message (msg)) == -1)
   {
     if (!(flags & SENDBATCH))
     {
+#ifdef _PGPPATH
+      if ((msg->pgp & PGPENCRYPT) || 
+	  ((msg->pgp & PGPSIGN) && msg->content->type == TYPEAPPLICATION))
+      {
+	mutt_free_body (&msg->content); /* destroy PGP data */
+	msg->content = clear_content;	/* restore clear text. */
+      }
+      else if ((msg->pgp & PGPSIGN) && msg->content->type == TYPEMULTIPART)
+      {
+	mutt_free_body (&msg->content->parts->next);		/* destroy sig */
+	msg->content = mutt_remove_multipart (msg->content);	/* remove multipart */
+      }
+#endif
       msg->content = mutt_remove_multipart (msg->content);
       goto main_loop;
     }
@@ -1359,6 +1373,15 @@ full_fcc:
   else if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
     mutt_message (i == 0 ? _("Mail sent.") : _("Sending in background."));
 
+#ifdef _PGPPATH
+  if (msg->pgp & PGPENCRYPT)
+  {
+    /* cleanup structures from the first encryption */
+    mutt_free_body (&clear_content);
+    FREE (&pgpkeylist);
+  }
+#endif /* _PGPPATH */
+
   if (flags & SENDREPLY)
   {
     if (cur)
